import click

import docker

from pydockenv import definitions
from pydockenv.client import Client
from pydockenv.commands.environment import create_env
from pydockenv.commands.environment import create_network
from pydockenv.commands.environment import get_current_env


def load(name, project_dir, input_file):
    click.echo(f'Loading environment {name} from {input_file}...')
    with open(input_file, 'rb') as fin:
        image = Client.get_instance().images.load(fin)[0]

    create_network(name)
    create_env(image, name, project_dir)

    click.echo(f'Environment {name} loaded from {input_file}!')


def save(name, output):
    current_env = get_current_env()

    click.echo(f'Saving environment {current_env}...')

    image_name = _commit(name, current_env)
    _export(image_name, output)

    click.echo(f'Removing image {image_name}...')
    Client.get_instance().images.remove(image_name)
    click.echo(f'Image {image_name} removed')


def _commit(name, current_env):
    click.echo(f'Saving environment {current_env} as image...')
    try:
        container = Client.get_instance().containers.get(
            definitions.CONTAINERS_PREFIX + current_env)
    except docker.errors.ImageNotFound:
        click.echo(f'Container {current_env} not found, exiting...')
        raise

    if not name:
        repository = f'{definitions.CONTAINERS_PREFIX + current_env}'
        tag = 'latest'
    else:
        repository, tag = name.split(':')

    container.commit(repository=repository, tag=tag)

    image_name = f'{repository}:{tag}'
    click.echo(f'Environment {current_env} saved as image {image_name}!')
    return image_name


def _export(image_name, output):
    click.echo(f'Saving image {image_name} to {output}...')

    try:
        image = Client.get_instance().images.get(image_name)
    except docker.errors.ImageNotFound:
        raise

    output = output or f'{image_name}.tar.gz'
    with open(output, 'wb') as fout:
        for chunk in image.save(named=True):
            fout.write(chunk)

    click.echo(f'Image {image_name} saved to {output}!')



import os
import subprocess
from contextlib import contextmanager
from itertools import chain

import click

import docker

from pydockenv import definitions
from pydockenv.client import Client
from pydockenv.commands.environment import get_current_env


class Executor:

    @classmethod
    def execute(cls, *args, **kwargs):
        client = Client.get_instance()
        current_env = get_current_env()
        try:
            container = client.containers.get(
                definitions.CONTAINERS_PREFIX + current_env)
        except docker.errors.NotFound:
            click.echo(f'Container {current_env} not found, exiting...')
            raise

        host_base_wd = container.labels['workdir']
        current_wd = os.getcwd()
        if not current_wd.startswith(host_base_wd):
            raise RuntimeError(
                f'Cannot run commands outside of {host_base_wd}')

        relative_wd = current_wd[len(host_base_wd):]
        guest_wd = f'/usr/src{relative_wd}'

        detach = kwargs.get('detach')
        env_vars = cls._build_env_vars(kwargs.get('env_vars'))
        with cls._with_mapped_ports(container, kwargs.get('ports'), detach):
            # This cannot be done with docker python sdk
            cmd = ['docker', 'exec', '-w', guest_wd]
            if detach:
                cmd.append('-d')
            else:
                cmd.extend(['-i', '-t'])

            cmd = (
                cmd + env_vars +
                [(definitions.CONTAINERS_PREFIX + current_env)] +
                list(args)
            )

            subprocess.check_call(cmd)

    @classmethod
    def _build_env_vars(cls, env_vars):
        if env_vars:
            return list(chain.from_iterable([
                ['-e', f'{k}={v}']for k, v in env_vars.items()
            ]))

        return []

    @classmethod
    @contextmanager
    def _with_mapped_ports(cls, container, ports, detach):
        if ports:
            port_mappers_containers_names = cls._run_port_mapper(
                container, ports)
        else:
            port_mappers_containers_names = []

        yield

        if detach:
            return

        for container_name in port_mappers_containers_names:
            container = Client.get_instance().containers.get(container_name)
            container.stop()

    @classmethod
    def _run_port_mapper(cls, container, ports):
        network_name = f'{container.name}_network'
        guest_ip = container.attrs['NetworkSettings']['Networks'][
            network_name]['IPAddress']
        containers_names = []
        for port in ports:
            # TODO: Use a single container for all port mappings instead of
            # spinning a container for each port
            name = f'{container.name}_port_mapper_{port}'
            client = Client.get_instance()

            try:
                container = client.containers.get(name)
            except docker.errors.NotFound:
                cmd = f'TCP-LISTEN:1234,fork TCP-CONNECT:{guest_ip}:{port}'
                kwargs = {
                    'command': cmd,
                    'ports': {'1234': f'{port}/tcp'},
                    'name': name,
                    'detach': True,
                    'auto_remove': True,
                    'network': network_name,
                }

                client.containers.run('alpine/socat', **kwargs)
            else:
                container.start()

            containers_names.append(name)

        return containers_names



import click

from pydockenv.commands import dependency
from pydockenv.commands import environment
from pydockenv.commands import io
from pydockenv.executor import Executor


@click.group()
def cli():
    pass


@cli.command()
@click.argument('name')
@click.argument('project_dir')
@click.option('--version', help='Python version')
def create(name, project_dir, version):
    environment.create(name, project_dir, version)


@cli.command()
def status():
    environment.status()


@cli.command()
@click.argument('name')
def activate(name):
    environment.activate(name)


@cli.command()
def deactivate():
    environment.deactivate()


@cli.command()
@click.argument('name')
def remove(name):
    environment.remove(name)


@cli.command()
def list_environments():
    environment.list_environments()


@cli.command()
@click.argument('package', required=False)
@click.option('-f', '--file', 'requirements_file',
              help='File to containing the requirements to install')
def install(package, requirements_file):
    dependency.install(package, requirements_file)


@cli.command()
@click.argument('package')
@click.option('-y', '--yes', is_flag=True)
def uninstall(package, yes):
    dependency.uninstall(package, yes)


@cli.command()
def list_packages():
    dependency.list_packages()


@cli.command()
@click.argument('name')
@click.argument('project_dir')
@click.argument('input_file')
def load(name, project_dir, input_file):
    io.load(name, project_dir, input_file)


@cli.command()
@click.option('--output', help='Name of the output file')
def save(name, output):
    io.save(name, output)


@cli.command()
@click.argument('args', nargs=-1)
def shell(args):
    click.echo('Running...')
    try:
        Executor.execute('python', *args)
    finally:
        click.echo('Exited!')


@cli.command()
@click.argument('cmd')
@click.argument('args', nargs=-1)
@click.option('-d', '--detach', is_flag=True)
@click.option('-e', '--env-var', multiple=True,
              help='Environment variable to set')
@click.option('-p', '--port', multiple=True,
              help='Port to reach')
def run(cmd, args, detach, env_var, port):
    click.echo('Running...')
    env_vars = dict(e.split('=') for e in env_var)
    try:
        Executor.execute(cmd, *args, detach=detach,
                         env_vars=env_vars, ports=list(port))
    finally:
        click.echo('Exited!')


if __name__ == '__main__':
    cli()



import os

from pydockenv import definitions
from tests.base import BaseIntegrationTest


class TestIntegrationOtherCommands(BaseIntegrationTest):

    def test_run(self):
        data = [
            {
                'env_name': self._env_name(),
                'proj_name': 'test-proj-1',
                'v': '3.7',
            },
            {
                'env_name': self._env_name(),
                'proj_name': 'test-proj-2',
                'v': '3.6',
            },
            {
                'env_name': self._env_name(),
                'proj_name': 'test-proj-3',
                'v': '2.7',
            },
        ]

        for d in data:
            proj_dir = self._create_project_dir(d['proj_name'])
            out = self._commander.run(
                f"create {d['env_name']} {str(proj_dir)} --version={d['v']}")
            self.assertEqual(out.returncode, 0)
            with self._commander.active_env(d['env_name']) as env:
                os.chdir(proj_dir)
                out = self._commander.run('run -- python --version', env=env)
                self.assertEqual(out.returncode, 0)
                self.assertIn(f"Python {d['v']}", out.stdout.decode('utf8'))

            os.chdir(definitions.ROOT_DIR)



import docker


class Client:

    _instance = None

    @classmethod
    def get_instance(cls):
        if cls._instance is None:
            cls._instance = docker.from_env()

        return cls._instance



from pathlib import Path

import docker

from pydockenv import definitions
from tests.base import BaseIntegrationTest


class TestIntegrationEnvironmentCommands(BaseIntegrationTest):

    def test_create(self):
        env_name = self._env_name()
        proj_name, py_version = 'test-proj', '3.7'

        cont_name = definitions.CONTAINERS_PREFIX + env_name

        with self.assertRaises(docker.errors.NotFound):
            self._client.containers.get(cont_name)

        proj_dir = self._create_project_dir(proj_name)
        out = self._commander.run(
            f'create {env_name} {str(proj_dir)} --version={py_version}')
        self.assertEqual(out.returncode, 0)

        expected = (f'Environment {env_name} with python version '
                    f'{py_version} created!')
        self.assertIn(expected, out.stdout.decode('utf8'))

        r = self._client.containers.get(cont_name)
        self.assertEqual(r.status, 'created')

        r = self._low_level_client.inspect_container(cont_name)
        self.assertEqual(len(r['Mounts']), 1)

        expected = {
            'Destination': '/usr/src',
            'Mode': '',
            'Propagation': 'rprivate',
            'RW': True,
            'Source': str(proj_dir.absolute()),
            'Type': 'bind'
        }
        actual = r['Mounts'][0]
        self.assertEqual(expected, actual)

        expected = {f'{cont_name}_network'}
        actual = set(r['NetworkSettings']['Networks'].keys())
        self.assertEqual(expected, actual)

        expected = {'workdir': str(Path(self._projs_dir, proj_name))}
        actual = r['Config']['Labels']
        self.assertEqual(expected, actual)

    def test_remove(self):
        env_name = self._env_name()
        proj_name, py_version = 'test-proj', '3.7'

        cont_name = definitions.CONTAINERS_PREFIX + env_name

        with self.assertRaises(docker.errors.NotFound):
            self._client.containers.get(cont_name)

        proj_dir = self._create_project_dir(proj_name)
        out = self._commander.run(
            f'create {env_name} {str(proj_dir)} --version={py_version}')
        self.assertEqual(out.returncode, 0)

        r = self._client.containers.get(cont_name)
        self.assertEqual(r.status, 'created')

        out = self._commander.run(f'remove {env_name}')
        self.assertEqual(out.returncode, 0)

        with self.assertRaises(docker.errors.NotFound):
            self._client.containers.get(cont_name)

        with self.assertRaises(docker.errors.NotFound):
            self._client.networks.get(f'{cont_name}_network')

    def test_activate(self):
        env_name = self._env_name()
        proj_name, py_version = 'test-proj', '3.7'
        proj_dir = self._create_project_dir(proj_name)

        out = self._commander.run(
            f'create {env_name} {str(proj_dir)} --version={py_version}')
        self.assertEqual(out.returncode, 0)
        env_diff = self._commander.activate_env(f'{env_name}')

        self.assertTrue({'PYDOCKENV', 'PYDOCKENV_DEBUG', 'PS1', 'SHLVL'} <=
                        set(env_diff.keys()))
        self.assertEqual(env_diff['PYDOCKENV'][1], env_name)
        self.assertEqual(env_diff['PYDOCKENV_DEBUG'][1], '1')
        self.assertEqual(env_diff['PS1'][1], f'({env_name})')
        self.assertEqual(int(env_diff['SHLVL'][1]),
                         int(env_diff['SHLVL'][0] or 0) + 1)

    def test_deactivate(self):
        env_name = self._env_name()
        proj_name, py_version = 'test-proj', '3.7'
        proj_dir = self._create_project_dir(proj_name)

        out = self._commander.run(
            f'create {env_name} {str(proj_dir)} --version={py_version}')
        self.assertEqual(out.returncode, 0)

        with self._commander.active_env(env_name) as env:
            env_diff_post_deactivate = self._commander.deactivate_env(env=env)

            self.assertEqual({'PYDOCKENV', 'PS1', 'SHLVL'},
                             env_diff_post_deactivate.keys())
            self.assertEqual(env_diff_post_deactivate['PYDOCKENV'][1], '')
            self.assertEqual(env_diff_post_deactivate['PS1'][1], '')
            self.assertEqual(int(env_diff_post_deactivate['SHLVL'][1]),
                             int(env_diff_post_deactivate['SHLVL'][0]) + 1)

    def test_list_environments(self):
        out = self._commander.run('list-environments')
        self.assertEqual(out.returncode, 0)

        stdout_lines = out.stdout.decode('utf8').split('\n')
        initial_envs = set([s.strip() for s in stdout_lines if s][1:-1])

        data = [
            {
                'env_name': self._env_name(),
                'proj_name': 'test-proj-1',
                'v': '3.7',
            },
            {
                'env_name': self._env_name(),
                'proj_name': 'test-proj-2',
                'v': '3.6',
            },
            {
                'env_name': self._env_name(),
                'proj_name': 'test-proj-3',
                'v': '2.7',
            },
        ]

        for d in data:
            with self.assertRaises(docker.errors.NotFound):
                self._client.containers.get(
                    definitions.CONTAINERS_PREFIX + d['env_name'])

            proj_dir = self._create_project_dir(d['proj_name'])
            out = self._commander.run(
                f"create {d['env_name']} {str(proj_dir)} --version={d['v']}"
            )
            self.assertEqual(out.returncode, 0)

        out = self._commander.run('list-environments')
        self.assertEqual(out.returncode, 0)

        stdout_lines = out.stdout.decode('utf8').split('\n')
        envs = set([s.strip() for s in stdout_lines if s][1:-1])

        self.assertEqual(envs - initial_envs, {d['env_name'] for d in data})

    def test_status(self):
        env_name = self._env_name()
        proj_name, py_version = 'test-proj', '3.7'
        proj_dir = self._create_project_dir(proj_name)

        out = self._commander.run(
            f'create {env_name} {str(proj_dir)} --version={py_version}')
        self.assertEqual(out.returncode, 0)

        out = self._commander.run('status')
        self.assertEqual(out.returncode, 0)
        self.assertEqual(out.stdout.decode('utf8').strip(),
                         'No active environment')

        with self._commander.active_env(env_name) as env:
            out = self._commander.run('status', env=env)
            self.assertEqual(out.returncode, 0)
            self.assertEqual(out.stdout.decode('utf8').strip(),
                             f'Active environment: {env_name}')



import os

from tests.base import BaseIntegrationTest


class TestIntegrationDependencyCommands(BaseIntegrationTest):

    def test_deps_handling(self):
        env_name = self._env_name()
        proj_name, py_version = 'test-proj', '3.7'

        proj_dir = self._create_project_dir(proj_name)
        out = self._commander.run(
            f'create {env_name} {str(proj_dir)} --version={py_version}')
        self.assertEqual(out.returncode, 0)
        with self._commander.active_env(env_name) as env:
            os.chdir(proj_dir)

            out = self._commander.run('list-packages', env=env)
            self.assertEqual(out.returncode, 0)
            self.assertNotIn(f'pydockenv', out.stdout.decode('utf8'))

            out = self._commander.run('install pydockenv', env=env)
            self.assertEqual(out.returncode, 0)

            out = self._commander.run('list-packages', env=env)
            self.assertEqual(out.returncode, 0)
            self.assertIn(f'pydockenv', out.stdout.decode('utf8'))

            out = self._commander.run('uninstall -y pydockenv ', env=env)
            self.assertEqual(out.returncode, 0)

            out = self._commander.run('list-packages', env=env)
            self.assertEqual(out.returncode, 0)
            self.assertNotIn(f'pydockenv', out.stdout.decode('utf8'))



import os

import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

import docker

from pydockenv import definitions
from pydockenv.client import Client
from tests.base import BaseIntegrationTest


class TestIntegrationPortMapperCommands(BaseIntegrationTest):

    def assertPortMapperExists(self, env_name, port):
        port_mapper_container_name = (
            definitions.CONTAINERS_PREFIX + env_name + f'_port_mapper_{port}'
        )

        try:
            Client.get_instance().containers.get(
                port_mapper_container_name)
        except docker.errors.NotFound:
            self.fail(
                f'Cannot find port mapper for environment {env_name} with'
                f'port {port}'
            )

    def test_port_mapping_single_port(self):
        env_name = self._env_name()
        proj_name, py_version = 'test-proj', '3.7'

        proj_dir = self._create_project_dir(proj_name)
        self._commander.run(
            f'create {env_name} {str(proj_dir)} --version={py_version}')
        with self._commander.active_env(env_name) as env:
            os.chdir(proj_dir)

            port = 8000
            out = self._commander.run(
                f'run -d -p {port} -- python -m http.server {port}', env=env)
            self.assertEqual(out.returncode, 0)
            self.assertPortMapperExists(env_name, port)

            s = requests.Session()
            s.mount('http://', HTTPAdapter(
                max_retries=Retry(connect=3, backoff_factor=1)))
            r = s.get(f'http://localhost:{port}')

            self.assertEqual(r.status_code, 200)

    def test_port_mapping_multi_ports(self):
        env_name = self._env_name()
        proj_name, py_version = 'test-proj', '3.7'

        proj_dir = self._create_project_dir(proj_name)
        self._commander.run(
            f'create {env_name} {str(proj_dir)} --version={py_version}')
        with self._commander.active_env(env_name) as env:
            os.chdir(proj_dir)

            for port in range(8000, 8003):
                out = self._commander.run(
                    f'run -d -p {port} -- python -m http.server {port}',
                    env=env
                )
                self.assertEqual(out.returncode, 0)
                self.assertPortMapperExists(env_name, port)

                s = requests.Session()
                s.mount('http://', HTTPAdapter(
                    max_retries=Retry(connect=3, backoff_factor=1)))
                r = s.get(f'http://localhost:{port}')

                self.assertEqual(r.status_code, 200)



import click

from pydockenv.executor import Executor


def install(package, requirements_file):
    args = ['pip', 'install']
    if requirements_file:
        args.extend(['-r', requirements_file])
    else:
        args.append(package)

    return Executor.execute(*args)


def uninstall(package, yes):
    click.echo('Running...')
    args = ['pip', 'uninstall']
    if yes:
        args.append('-y')

    args.append(package)
    try:
        Executor.execute(*args)
    finally:
        click.echo('Exited!')


def list_packages():
    click.echo('Running...')
    try:
        Executor.execute('pip', 'freeze')
    finally:
        click.echo('Exited!')



import re
from functools import partial

import setuptools


requires = [
    'Click>=7.0,<8.0',
    'docker>=3.7.0,<3.8.0',
]

scripts = [
    'bin/pydockenv',
]

classifiers = [
    'Development Status :: 3 - Alpha',
    'Environment :: Console',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: Apache Software License',
    'Operating System :: Unix',
    'Programming Language :: Unix Shell',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Topic :: Software Development',
    'Topic :: Utilities'
]

with open('README.md', 'r') as fh:
    long_description = fh.read()


def get_about():
    regexes = {
        'title': r"^__title__\s=\s(?P<quote>['])(?P<title>\w*)(?P=quote)$",
        'version': r"^__version__\s=\s(?P<quote>['])(?P<version>[\d\.]*)(?P=quote)$",
        'author': r"^__author__\s=\s(?P<quote>['])(?P<author>[\w\s]*)(?P=quote)$",
        'author_email': r"^__author_email__\s=\s(?P<quote>['])(?P<author_email>.*)(?P=quote)$",
        'description': r"^__description__\s=\s(?P<quote>['])(?P<description>.*)(?P=quote)$",
        'project_url': r"^__project_url__\s=\s(?P<quote>['])(?P<project_url>.*)(?P=quote)$",
    }

    with open('./pydockenv/__init__.py') as f:
        raw_about = f.read()

    extract = partial(re.search, string=raw_about, flags=re.MULTILINE)
    return {k: extract(v).group(k) for k, v in regexes.items()}


about = get_about()


setuptools.setup(
    name=about['title'],
    version=about['version'],
    author=about['author'],
    author_email=about['author_email'],
    description=about['description'],
    long_description=long_description,
    long_description_content_type='text/markdown',
    url=about['project_url'],
    packages=setuptools.find_packages(exclude=['tests']),
    scripts=scripts,
    package_data={'': ['LICENSE']},
    include_package_data=True,
    install_requires=requires,
    classifiers=classifiers,
)



import os
import subprocess
from contextlib import contextmanager
from pathlib import Path

from pydockenv import definitions


BIN_PATH = str(Path(definitions.ROOT_DIR, 'bin', 'pydockenv'))


class Commander:

    _instance = None

    def __init__(self, env=None):
        self._bin_path = BIN_PATH
        self._env = env or {}

    @classmethod
    def get_instance(cls):
        if cls._instance is None:
            cls._instance = Commander()

        return cls._instance

    def add_env_var(self, k, v):
        self._env[k] = v

    def run(self, cmd, env=None):
        args = cmd.split(' ')

        env = self._prepare_env(env)

        return subprocess.run(
            [self._bin_path, *args],
            stdout=subprocess.PIPE, stderr=subprocess.PIPE,
            env=env
        )

    @contextmanager
    def active_env(self, env_name):
        env_diff = self.activate_env(env_name)
        env = os.environ.copy()
        env.update({k: v[1] for k, v in env_diff.items()})

        try:
            yield env
        finally:
            self.deactivate_env(env=env)

    def activate_env(self, env_name, env=None):
        return self.source(f'activate {env_name}', env=env)

    def deactivate_env(self, env=None):
        return self.source('deactivate', env=env)

    def source(self, cmd, env=None):
        env = self._prepare_env(env)

        proc = subprocess.Popen('env', stdout=subprocess.PIPE, shell=True,
                                env=env)
        initial_env = self._get_env(proc.stdout)
        proc.communicate()

        command = f"bash -c 'PYDOCKENV_DEBUG=1 source {self._bin_path} {cmd}'"
        proc = subprocess.Popen(command, stdout=subprocess.DEVNULL,
                                stderr=subprocess.PIPE, shell=True, env=env)
        post_env = self._get_env(proc.stderr)
        proc.communicate()

        env_diff = {}
        for k in set().union(initial_env.keys(), post_env.keys()):
            initial_value, post_value = initial_env.get(k), post_env.get(k)
            if initial_value != post_value:
                env_diff[k] = (initial_value, post_value)

        return env_diff

    def _get_env(self, stdout):
        env = {}
        for line in stdout:
            (key, _, value) = line.decode('utf8').strip().partition("=")
            env[key] = value

        return env

    def _prepare_env(self, env):
        env = {**self._env, **(env or {})}
        env['PYTHONPATH'] = definitions.ROOT_DIR
        if env:
            env = {k: v for k, v in {**os.environ, **env}.items()
                   if v is not None}
        else:
            env = None

        return env



__title__ = 'pydockenv'
__version__ = '0.3.1'
__author__ = 'Lou Marvin Caraig'
__author_email__ = 'loumarvincaraig@gmail.com'
__description__ = 'Python Virtualenv Powered by Docker'
__project_url__ = 'https://github.com/se7entyse7en/pydockenv'
__copyright__ = 'Copyright 2019 Lou Marvin Caraig'



import os

import click

import docker
from docker.types import Mount

from pydockenv import definitions
from pydockenv.client import Client


def get_current_env():
    return os.environ.get('PYDOCKENV')


def create(name, project_dir, version):
    version = version or 'latest'
    click.echo(f'Creating environment {name} with python version {version}...')
    image_name = f'python:{version}'

    client = Client.get_instance()
    try:
        image = client.images.get(image_name)
    except docker.errors.ImageNotFound:
        click.echo(f'Image {image_name} not found, pulling...')
        image = client.images.pull('python', tag=version)

    create_network(name)
    create_env(image, name, project_dir)

    click.echo(f'Environment {name} with python version {version} created!')


def status():
    current_env = get_current_env()
    if not current_env:
        click.echo('No active environment')
    else:
        click.echo(f'Active environment: {current_env}')


def activate(name):
    click.echo('Activating environment...')
    try:
        container = Client.get_instance().containers.get(
            definitions.CONTAINERS_PREFIX + name)
    except docker.errors.NotFound:
        click.echo(f'Environment {name} not found, exiting...')
    else:
        container.start()
        click.echo('Environment activated!')


def deactivate():
    click.echo('Deactivating current environment...')
    current_env = get_current_env()
    try:
        container = Client.get_instance().containers.get(
            definitions.CONTAINERS_PREFIX + current_env)
    except docker.errors.ImageNotFound:
        click.echo(f'Environment {current_env} not found, exiting...')
    else:
        container.stop()
        click.echo('Environment deactivated!')


def remove(name):
    click.echo(f'Removing environment {name}...')
    try:
        container = Client.get_instance().containers.get(
            definitions.CONTAINERS_PREFIX + name)
    except docker.errors.NotFound:
        click.echo(f'Environment {name} not found, exiting...')
        raise

    kwargs = {
        'force': True,
    }
    container.remove(**kwargs)
    delete_network(name)
    click.echo(f'Environment {name} removed!')


def list_environments():
    click.echo(f'Listing environments...')
    kwargs = {
        'all': True,
    }
    containers = Client.get_instance().containers.list(kwargs)

    current_env = get_current_env()
    envs = []
    for c in containers:
        if not c.name.startswith(definitions.CONTAINERS_PREFIX):
            continue

        env_name = c.name[len(definitions.CONTAINERS_PREFIX):]
        prefix = '* ' if env_name == current_env else '  '
        envs.append(f'{prefix}{env_name}')

    click.echo('\n'.join(envs))
    click.echo(f'Environments listed!')


def create_network(env_name):
    network_name = definitions.CONTAINERS_PREFIX + env_name + '_network'
    Client.get_instance().networks.create(network_name, check_duplicate=True)


def delete_network(env_name):
    network_name = definitions.CONTAINERS_PREFIX + env_name + '_network'
    try:
        network = Client.get_instance().networks.get(network_name)
    except docker.errors.ImageNotFound:
        click.echo(f'Network {network_name} not found, exiting...')
        raise

    for c in network.containers:
        network.disconnect(c)

    network.remove()


def create_env(image, name, project_dir):
    workdir = os.path.abspath(project_dir)
    mounts = [
        Mount('/usr/src', workdir, type='bind')
    ]
    kwargs = {
        'command': '/bin/sh',
        'stdin_open': True,
        'labels': {
            'workdir': workdir
        },
        'name': definitions.CONTAINERS_PREFIX + name,
        'mounts': mounts,
        'network': definitions.CONTAINERS_PREFIX + name + '_network',
    }
    Client.get_instance().containers.create(image, **kwargs)
